@@ -18,15 +18,13 @@
 #include "ntp_config.h"
 #include "ntp_cmdargs.h"
 
-#ifdef PUBKEY
+#ifdef OPENSSL
 # include "ntp_crypto.h"
-#endif /* PUBKEY */
+#endif /* OPENSSL */
 
 #include <stdio.h>
 #include <ctype.h>
-#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
-#endif
 #include <signal.h>
 #ifndef SIGCHLD
 # define SIGCHLD SIGCLD
@@ -81,8 +79,8 @@ extern int priority_done;
  * setvar [ ]
  * logfile logfile
  * logconfig [+|-|=][{sync|sys|peer|clock}{{,all}{info|statistics|events|status}}]...
- * enable auth|bclient|pll|kernel|monitor|stats|calibrate
- * disable auth|bclient|pll|kernel|monitor|stats|calibrate
+ * enable auth|bclient|pll|kernel|monitor|stats
+ * disable auth|bclient|pll|kernel|monitor|stats
  * phone ...
  * pps device [assert|clear] [hardpps]
  * priority high|normal
@@ -107,9 +105,9 @@ static	struct keyword keywords[] = {
 	{ "broadcastdelay",	CONFIG_BDELAY },
 	{ "clientlimit",	CONFIG_CLIENTLIMIT },
 	{ "clientperiod",	CONFIG_CLIENTPERIOD },
-#ifdef PUBKEY
+#ifdef OPENSSL
 	{ "crypto",		CONFIG_CRYPTO },
-#endif /* PUBKEY */
+#endif /* OPENSSL */
 	{ "controlkey",		CONFIG_CONTROLKEY },
 	{ "disable",		CONFIG_DISABLE },
 	{ "driftfile",		CONFIG_DRIFTFILE },
@@ -118,9 +116,9 @@ static	struct keyword keywords[] = {
 	{ "fudge",		CONFIG_FUDGE },
 	{ "includefile",	CONFIG_INCLUDEFILE },
 	{ "keys",		CONFIG_KEYS },
-#ifdef PUBKEY
+#ifdef OPENSSL
 	{ "keysdir",		CONFIG_KEYSDIR },
-#endif /* PUBKEY */
+#endif /* OPENSSL */
 	{ "logconfig",		CONFIG_LOGCONFIG },
 	{ "logfile",		CONFIG_LOGFILE },
 	{ "manycastclient",	CONFIG_MANYCASTCLIENT },
@@ -156,9 +154,9 @@ static	struct keyword mod_keywords[] = {
 	{ "mode",		CONF_MOD_MODE },    /* refclocks */
 	{ "noselect",		CONF_MOD_NOSELECT },
 	{ "prefer",		CONF_MOD_PREFER },
-#ifdef PUBKEY
-	{ "publickey",		CONF_MOD_PUBLICKEY },
-#endif /* PUBKEY */
+#ifdef OPENSSL
+	{ "certificate",	CONF_MOD_CERT },
+#endif /* OPENSSL */
 	{ "ttl",		CONF_MOD_TTL },     /* NTP peers */
 	{ "version",		CONF_MOD_VERSION },
 	{ "",			CONFIG_UNKNOWN }
@@ -242,12 +240,12 @@ static	struct keyword fgen_types[] = {
 static struct keyword flags_keywords[] = {
 	{ "auth",		PROTO_AUTHENTICATE },
 	{ "bclient",		PROTO_BROADCLIENT },
-	{ "calibrate",		PROTO_CAL },
 	{ "kernel",		PROTO_KERNEL },
 	{ "monitor",		PROTO_MONITOR },
 	{ "ntp",		PROTO_NTP },
-	{ "pps",		PROTO_PPS },
 	{ "stats",		PROTO_FILEGEN },
+	{ "pps",		PROTO_PPS },
+	{ "calibrate",		PROTO_CAL },
 	{ "",			CONFIG_UNKNOWN }
 };
 
@@ -275,19 +273,19 @@ static struct keyword tinker_keywords[] = {
 	{ "",			CONFIG_UNKNOWN }
 };
 
-#ifdef PUBKEY
+#ifdef OPENSSL
 /*
  * "crypto" modifier keywords
  */
 static struct keyword crypto_keywords[] = {
-	{ "dh",			CONF_CRYPTO_DH },
-	{ "flags",		CONF_CRYPTO_FLAGS },
+	{ "signkey",		CONF_CRYPTO_SIGN },
 	{ "leap",		CONF_CRYPTO_LEAP },
-	{ "privatekey",		CONF_CRYPTO_PRIVATEKEY },
-	{ "publickey",		CONF_CRYPTO_PUBLICKEY },
+	{ "rsakey",		CONF_CRYPTO_RSA },
+	{ "certificate",	CONF_CRYPTO_CERT },
+	{ "randfile",		CONF_CRYPTO_RAND },
 	{ "",			CONFIG_UNKNOWN }
 };
-#endif /* PUBKEY */
+#endif /* OPENSSL */
 
 /*
  * "logconfig" building blocks
@@ -671,7 +669,7 @@ getconfig(
 			minpoll = NTP_MINDPOLL;
 			maxpoll = NTP_MAXDPOLL;
 			peerkey = 0;
-			peerkeystr = (u_char *)"*";
+			peerkeystr = "*";
 			peerflags = 0;
 			ttl = 0;
 			for (i = 2; i < ntokens; i++)
@@ -752,14 +750,13 @@ getconfig(
 				case CONF_MOD_IBURST:
 				    peerflags |= FLAG_IBURST;
 				    break;
-#ifdef AUTOKEY
+#ifdef OPENSSL
 				case CONF_MOD_SKEY:
 				    peerflags |= FLAG_SKEY |
 					FLAG_AUTHENABLE;
 				    break;
 
-#ifdef PUBKEY
-				case CONF_MOD_PUBLICKEY:
+				case CONF_MOD_CERT:
 				    if (i >= ntokens - 1) {
 					msyslog(LOG_ERR,
 					    "Public key file name required");
@@ -770,8 +767,7 @@ getconfig(
 					FLAG_AUTHENABLE;
  				    peerkeystr = tokens[++i];
 				    break;
-#endif /* PUBKEY */
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 				case CONF_MOD_TTL:
 				    if (i >= ntokens-1) {
@@ -809,6 +805,9 @@ getconfig(
 						"configuration of %s failed",
 						ntoa(&peeraddr));
 			    }
+			    if (tok == CONFIG_MANYCASTCLIENT)
+				proto_config(PROTO_MULTICAST_ADD,
+				    peeraddr.sin_addr.s_addr, 0.);
 	
 			} else if (errflg == -1) {
 				save_resolve(tokens[1], hmode, peerversion,
@@ -1000,10 +999,10 @@ getconfig(
 			}
 			break;
 
-#ifdef AUTOKEY
+#ifdef OPENSSL
 		    case CONFIG_REVOKE:
 			if (ntokens >= 2)
-			    sys_revoke = 1 << max(atoi(tokens[1]), 10);
+			    sys_revoke = max(atoi(tokens[1]), KEY_REVOKE);
 			break;
 
 		    case CONFIG_AUTOMAX:
@@ -1011,7 +1010,6 @@ getconfig(
 			    sys_automax = 1 << max(atoi(tokens[1]), 10);
 			break;
 
-#ifdef PUBKEY
 		    case CONFIG_KEYSDIR:
 			if (ntokens < 2) {
 			    msyslog(LOG_ERR,
@@ -1023,7 +1021,7 @@ getconfig(
 	
 		    case CONFIG_CRYPTO:
 			if (ntokens == 1) {
-				crypto_config(CRYPTO_CONF_FLAGS	, "0");
+				crypto_config(CRYPTO_CONF_NONE, NULL);
 				break;
 			}
 			for (i = 1; i < ntokens; i++) {
@@ -1037,38 +1035,33 @@ getconfig(
 				break;
 			    }
 			    switch(temp) {
-			    case CONF_CRYPTO_FLAGS:
-				crypto_config(CRYPTO_CONF_FLAGS, tokens[i]);
-				break;
-
 			    case CONF_CRYPTO_LEAP:
 				crypto_config(CRYPTO_CONF_LEAP, tokens[i]);
 				break;
 
-			    case CONF_CRYPTO_DH:
-				crypto_config(CRYPTO_CONF_DH, tokens[i]);
-				break;
-
-			    case CONF_CRYPTO_PRIVATEKEY:
+			    case CONF_CRYPTO_RSA:
 				crypto_config(CRYPTO_CONF_PRIV, tokens[i]);
 				break;
 
-			    case CONF_CRYPTO_PUBLICKEY:
-				crypto_config(CRYPTO_CONF_PUBL, tokens[i]);
+			    case CONF_CRYPTO_SIGN:
+				crypto_config(CRYPTO_CONF_SIGN, tokens[i]);
 				break;
 
 			    case CONF_CRYPTO_CERT:
 				crypto_config(CRYPTO_CONF_CERT, tokens[i]);
 				break;
 
+			    case CONF_CRYPTO_RAND:
+				crypto_config(CRYPTO_CONF_RAND, tokens[i]);
+				break;
+
 			    default:
 				msyslog(LOG_ERR, "crypto: unknown keyword");
 				break;
 			    }
 			}
 			break;
-#endif /* PUBKEY */
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 		    case CONFIG_RESTRICT:
 			if (ntokens < 2) {
@@ -1801,7 +1794,7 @@ gettokens_netinfo (
 				for (index = 0; index < namelist.ni_namelist_len; index++) {
 					char *value = namelist.ni_namelist_val[index];
 
-					if (! (val_list[index] = (char*)malloc(strlen(value)+1)))
+					if (! (val_list[index] = (char*)malloc(strlen(value+1))))
 						{ msyslog(LOG_ERR, "out of memory while configuring"); break; }
 
 					strcpy(val_list[index], value);
@@ -2006,10 +1999,7 @@ getnetnum(
 #ifdef DEBUG
 		if (debug > 3)
 			printf(
-				"getnetnum: \"%s\" invalid host number%s.\n",
-				(complain)
-				? ", line ignored"
-				: "",
+				"getnetnum: \"%s\" invalid host number, line ignored\n",
 				num);
 #endif
 		return 0;
@@ -2106,11 +2096,11 @@ save_resolve(
 	}
 #endif
 
-	(void)fprintf(res_fp, "%s %d %d %d %d %d %d %u %s\n", name,
+	(void)fprintf(res_fp, "%s %d %d %d %d %d %d %d %s\n", name,
 	    mode, version, minpoll, maxpoll, flags, ttl, keyid, keystr);
 #ifdef DEBUG
 	if (debug > 1)
-		printf("config: %s %d %d %d %d %x %d %u %s\n", name, mode,
+		printf("config: %s %d %d %d %d %x %d %08x %s\n", name, mode,
 		    version, minpoll, maxpoll, flags, ttl, keyid, keystr);
 #endif
 
@@ -2214,7 +2204,7 @@ do_resolve_internal(void)
 		 */
 
 		closelog();
-		kill_asyncio(0);
+		kill_asyncio();
 
 		(void) signal_no_reset(SIGCHLD, SIG_DFL);
 
