@@ -26,13 +26,9 @@
 #ifdef HAVE_NETINET_IN_SYSTM_H
 # include <netinet/in_systm.h>
 #else /* Some old linux systems at least have in_system.h instead. */
-# ifdef HAVE_NETINET_IN_SYSTEM_H
-#  include <netinet/in_system.h>
-# endif
+# include <netinet/in_system.h>
 #endif /* HAVE_NETINET_IN_SYSTM_H */
-#ifdef HAVE_NETINET_IP_H
-# include <netinet/ip.h>
-#endif
+#include <netinet/ip.h>
 #ifdef HAVE_SYS_IOCTL_H
 # include <sys/ioctl.h>
 #endif
@@ -41,8 +37,6 @@
 #endif
 #include <arpa/inet.h>
 
-extern int listen_to_virtual_ips;
-
 #if _BSDI_VERSION >= 199510
 # include <ifaddrs.h>
 #endif
@@ -142,7 +136,6 @@ static	int open_socket		P((struct sockaddr_in *, int, int));
 static	void	close_socket	P((int));
 static	void	close_file	P((int));
 static	char *	fdbits		P((int, fd_set *));
-static	void	set_reuseaddr	P((int));
 
 /*
  * init_io - initialize I/O data structures and call socket creation routine
@@ -278,15 +271,9 @@ create_sockets(
 		if ((ifap->ifa_flags & IFF_UP) == 0)
 		    continue;
 
-		if (debug)
-			printf("after getifaddrs(), considering %s (%s)\n",
-			       ifap->ifa_name,
-			       inet_ntoa(((struct sockaddr_in *)ifap->ifa_addr)->sin_addr));
-
 		if (ifap->ifa_flags & IFF_LOOPBACK) {
 			sin = (struct sockaddr_in *)ifap->ifa_addr;
-			if (ntohl(sin->sin_addr.s_addr) != 0x7f000001 &&
-			    !listen_to_virtual_ips)
+			if (ntohl(sin->sin_addr.s_addr) != 0x7f000001)
 				continue;
 		}
 		inter_list[i].flags = 0;
@@ -327,10 +314,10 @@ create_sockets(
 		 * one physical interface. -wsr
 		 */
 		for (j=0; j < i; j++)
-		    if ((inter_list[j].sin.sin_addr.s_addr &
-			 inter_list[j].mask.sin_addr.s_addr) ==
-			(inter_list[i].sin.sin_addr.s_addr &
-			 inter_list[i].mask.sin_addr.s_addr))
+		    if (inter_list[j].sin.sin_addr.s_addr &
+			inter_list[j].mask.sin_addr.s_addr ==
+			inter_list[i].sin.sin_addr.s_addr &
+			inter_list[i].mask.sin_addr.s_addr)
 		    {
 			    if (inter_list[j].flags & INT_LOOPBACK)
 				inter_list[j] = inter_list[i];
@@ -404,6 +391,8 @@ create_sockets(
 	for(n = ifc.ifc_len, ifr = ifc.ifc_req; n > 0;
 	    ifr = (struct ifreq *)((char *)ifr + size))
 	{
+		extern int listen_to_virtual_ips;
+
 		size = sizeof(*ifr);
 
 # ifdef HAVE_SA_LEN_IN_STRUCT_SOCKADDR
@@ -476,7 +465,7 @@ create_sockets(
 			(ifreq.ifr_flags & IFF_LOOPBACK)
 #  else /* not IFF_LOCAL_LOOPBACK and not IFF_LOOPBACK */
 			/* test against 127.0.0.1 (yuck!!) */
-			((*(struct sockaddr_in *)&ifr->ifr_addr).sin_addr.s_addr == inet_addr("127.0.0.1"))
+			(inter_list[i].sin.sin_addr.s_addr == inet_addr("127.0.0.1"))
 #  endif /* not IFF_LOCAL_LOOPBACK and not IFF_LOOPBACK */
 			)
 		{
@@ -616,7 +605,21 @@ create_sockets(
 	 * Now that we have opened all the sockets, turn off the reuse flag for
 	 * security.
 	 */
-	set_reuseaddr(0);
+	for (i = 0; i < ninterfaces; i++) {
+		int off = 0;
+
+		/*
+		 * if inter_list[ n ].fd  is -1, we might have a adapter
+		 * configured but not present
+		 */
+		if ( inter_list[ i ].fd != -1 ) {
+			if (setsockopt(inter_list[i].fd, SOL_SOCKET,
+				       SO_REUSEADDR, (char *)&off,
+				       sizeof(off))) {
+				msyslog(LOG_ERR, "create_sockets: setsockopt(SO_REUSEADDR,off) failed: %m");
+			}
+		}
+	}
 
 #if defined(MCAST)
 	/*
@@ -671,10 +674,6 @@ io_setbclient(void)
 {
 	int i;
 
-#ifdef OPEN_BCAST_SOCKET
-	set_reuseaddr(1);
-#endif
-
 	for (i = 1; i < ninterfaces; i++) {
 		if (!(inter_list[i].flags & INT_BROADCAST))
 			continue;
@@ -691,34 +690,6 @@ io_setbclient(void)
 		inter_list[i].flags |= INT_BCASTOPEN;
 #endif
 	}
-
-#ifdef OPEN_BCAST_SOCKET
-	set_reuseaddr(0);
-#endif
-}
-
-/*
- * set_reuseaddr() - set/clear REUSEADDR on all sockets
- *		NB possible hole - should we be doing this on broadcast
- *			fd's also?
- */
-static void
-set_reuseaddr(int flag) {
-	int i;
-
-	for (i = 0; i < ninterfaces; i++) {
-		/*
-		 * if inter_list[ n ].fd  is -1, we might have a adapter
-		 * configured but not present
-		 */
-		if ( inter_list[ i ].fd != -1 ) {
-			if (setsockopt(inter_list[i].fd, SOL_SOCKET,
-				       SO_REUSEADDR, (char *)&flag,
-				       sizeof(flag))) {
-				msyslog(LOG_ERR, "create_sockets: setsockopt(SO_REUSEADDR,%s) failed: %m", flag ? "on" : "off");
-			}
-		}
-	}
 }
 
 
@@ -766,9 +737,7 @@ io_multicast_add(
 	 * Try opening a socket for the specified class D address. This
 	 * works under SunOS 4.x, but not OSF1 .. :-(
 	 */
-	set_reuseaddr(1);
 	s = open_socket(sinp, 0, 1);
-	set_reuseaddr(0);
 	if (s < 0) {
 		memset((char *)&inter_list[i], 0, sizeof inter_list[0]);
 		i = 0;
@@ -799,6 +768,10 @@ io_multicast_add(
 	inter_list[i].flags |= INT_MULTICAST;
 	if (i >= ninterfaces)
 	    ninterfaces = i+1;
+#ifdef DEBUG
+	if (debug)
+		printf("io_multicast_add %s\n", inet_ntoa(iaddr));
+#endif
 #else /* MCAST */
 	struct in_addr iaddr;
 
@@ -1262,7 +1235,7 @@ fdbits(
 /*
  * input_handler - receive packets asynchronously
  */
-void
+extern void
 input_handler(
 	l_fp *cts
 	)
@@ -1537,11 +1510,8 @@ input_handler(
 			/*
 			 * extended FAU debugging output
 			 */
-			if (err != EINTR)
-			    msyslog(LOG_ERR,
-				    "select(%d, %s, 0L, 0L, &0.0) error: %m",
-				    maxactivefd+1,
-				    fdbits(maxactivefd, &activefds));
+			msyslog(LOG_ERR, "select(%d, %s, 0L, 0L, &0.000000) error: %m",
+				maxactivefd+1, fdbits(maxactivefd, &activefds));
 			if (err == EBADF) {
 				int j, b;
 
@@ -1619,7 +1589,7 @@ findbcastinter(
 	struct sockaddr_in *addr
 	)
 {
-#if !defined(MPE) && (defined(SIOCGIFCONF) || defined(SYS_WINNT))
+#if defined(SIOCGIFCONF) || defined(SYS_WINNT)
 	register int i;
 	register u_int32 xaddr;
 
@@ -1776,13 +1746,11 @@ io_closeclock(
 #endif	/* REFCLOCK */
 
 void
-kill_asyncio(
-	int startfd
-	)
+kill_asyncio(void)
 {
 	int i;
 
 	BLOCKIO();
-	for (i = startfd; i <= maxactivefd; i++)
+	for (i = 0; i <= maxactivefd; i++)
 	    (void)close_socket(i);
 }
