@@ -456,7 +456,7 @@ local_clock(
 		 * and ignore it.
 		 */
 		default:
-			allow_panic = FALSE;
+			allow_panic = TRUE;
 			if (fabs(fp_offset - last_offset) >
 			    CLOCK_SGATE * oerror && mu <
 			    ULOGTOD(sys_poll + 1)) {
@@ -497,7 +497,8 @@ local_clock(
 				i = 6;
 			etemp = fll[i];
 			dtemp = max(mu, ULOGTOD(allan_xpt));
-			flladj = (fp_offset - clock_offset) * etemp /
+			flladj = ((fp_offset - last_offset) +
+			    (fp_offset - clock_offset)) * etemp /
 			    (dtemp * CLOCK_FLL);
 			dtemp = ULOGTOD(SHIFT_PLL + 2 + sys_poll);
 			etemp = min(mu, ULOGTOD(sys_poll));
@@ -695,8 +696,8 @@ local_clock(
 	if (debug)
 		printf(
 		    "local_clock: mu %.0f noi %.3f stb %.3f pol %d cnt %d\n",
-		    mu, sys_jitter * 1e6, clock_stability * 1e6, sys_poll,
-		    tc_counter);
+		    mu, sys_jitter * 1e6 / mu, clock_stability * 1e6,
+		    sys_poll, tc_counter);
 #endif /* DEBUG */
 	return (retval);
 }
