@@ -10,9 +10,9 @@
 
 #include "ntpd.h"
 #include "ntp_stdlib.h"
-#ifdef AUTOKEY
+#ifdef OPENSSL
 #include "ntp_crypto.h"
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 /*
  *                  Table of valid association combinations
@@ -331,7 +331,18 @@ unpeer(
 	)
 {
 	int hash;
+	u_char	statstr[NTP_MAXSTRLEN]; /* statistics for filegen */
 
+#ifdef OPENSSL
+	if (peer_to_remove->flags & FLAG_SKEY) {
+		sprintf(statstr, "unpeer %d", peer_to_remove->associd);
+		record_crypto_stats(&peer_to_remove->srcadr, statstr);
+#ifdef DEBUG
+		if (debug)
+			printf("peer: %s\n", statstr);
+#endif
+	}
+#endif /* OPENSSL */
 	peer_associations--;
 #ifdef DEBUG
 	if (debug)
@@ -467,13 +478,13 @@ peer_config(
 		peer->version = (u_char)version;
 		peer->minpoll = (u_char)minpoll;
 		peer->maxpoll = (u_char)maxpoll;
-		peer->hpoll = peer->kpoll = peer->minpoll;
-		peer->ppoll = peer->maxpoll;
 		peer->flags = flags | FLAG_CONFIG |
 			(peer->flags & FLAG_REFCLOCK);
 		peer->cast_flags = cast_flags;
 		peer->ttlmax = ttl;
 		peer->keyid = key;
+		peer->precision = sys_precision;
+		peer_clear(peer);
 		return (peer);
 	}
 
@@ -507,6 +518,7 @@ newpeer(
 {
 	register struct peer *peer;
 	register int i;
+	u_char	statstr[NTP_MAXSTRLEN]; /* statistics for filegen */
 
 	/*
 	 * Allocate a new peer structure. Some dirt here, since some of
@@ -594,10 +606,20 @@ newpeer(
 	peer->ass_next = assoc_hash[i];
 	assoc_hash[i] = peer;
 	assoc_hash_count[i]++;
+#ifdef OPENSSL
+	if (peer->flags & FLAG_SKEY) {
+		sprintf(statstr, "newpeer %d", peer->associd);
+		record_crypto_stats(&peer->srcadr, statstr);
+#ifdef DEBUG
+		if (debug)
+			printf("peer: %s\n", statstr);
+#endif
+	}
+#endif /* OPENSSL */
 #ifdef DEBUG
 	if (debug)
 		printf(
-		    "newpeer: %s->%s mode %d vers %d poll %d %d flags %x %x ttl %d key %08x\n",
+		    "newpeer: %s->%s mode %d vers %d poll %d %d flags 0x%x 0x%x ttl %d key %08x\n",
 		    ntoa(&peer->dstadr->sin), ntoa(&peer->srcadr),
 		    peer->hmode, peer->version, peer->minpoll,
 		    peer->maxpoll, peer->flags, peer->cast_flags,
@@ -700,7 +722,7 @@ peer_all_reset(void)
 }
 
 
-#ifdef AUTOKEY
+#ifdef OPENSSL
 /*
  * expire_all - flush all crypto data and update timestamps.
  */
@@ -718,30 +740,27 @@ expire_all(void)
 	 * everything. Then, recompute and sign the agreement public
 	 * value, if present.
 	 */
+	if (!crypto_flags)
+		return;
 	for (n = 0; n < HASH_SIZE; n++) {
 		for (peer = peer_hash[n]; peer != 0; peer = next_peer) {
 			next_peer = peer->next;
 			if (peer->cast_flags & MDF_ACAST) {
 				peer_clear(peer);
-#ifdef AUTOKEY
-			} else {
+			} else if (peer->hmode == MODE_ACTIVE ||
+			    peer->hmode == MODE_PASSIVE) {
 				key_expire(peer);
-				peer->pcookie.tstamp = 0;
-#endif /* AUTOKEY */
+				peer->cookval.tstamp = 0;
+				peer->crypto &= ~(CRYPTO_FLAG_AUTO |
+				    CRYPTO_FLAG_AGREE);
 			}
 				
 		}
 	}
 	sys_private = (u_int32)RANDOM & 0xffffffff;
-#ifdef PUBKEY
-	crypto_agree();
-#endif /* PUBKEY */
-#ifdef DEBUG
-	if (debug)
-		printf("expire_all: at %lu\n", current_time);
-#endif
+	crypto_sign();
 }
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 
 /*
@@ -804,7 +823,7 @@ resetmanycast(void)
 		    peer->next) {
 			if (peer->cast_flags & MDF_ACAST) {
 				peer->ttl = 0;
-				poll_update(peer, peer->hpoll);
+				poll_update(peer, 0);
 			}
 		}
 	}
