@@ -36,9 +36,9 @@ static	double sys_offset;	/* current local clock offset */
 l_fp	sys_reftime;		/* time we were last updated */
 struct	peer *sys_peer; 	/* our current peer */
 struct	peer *sys_prefer;	/* our cherished peer */
-#ifdef AUTOKEY
+#ifdef OPENSSL
 u_long	sys_automax;		/* maximum session key lifetime */
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 /*
  * Nonspecified system state variables.
@@ -55,9 +55,9 @@ keyid_t	sys_private;		/* private value for session seed */
 int	sys_manycastserver;	/* respond to manycast client pkts */
 u_int sys_survivors;		/* truest of the truechimers */
 int	peer_ntpdate;		/* active peers in ntpdate mode */
-#ifdef AUTOKEY
+#ifdef OPENSSL
 char	*sys_hostname;		/* gethostname() name */
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 /*
  * Statistics counters
@@ -109,7 +109,7 @@ transmit(
 			 */
 			hpoll = peer->minpoll;
 			peer->ttl = peer->ttlmax;
-#ifdef AUTOKEY
+#ifdef OPENSSL
 		} else if (peer->cast_flags & MDF_ACAST) {
 
 			/*
@@ -135,7 +135,7 @@ transmit(
 			    peer->ttlmax)
 				peer->ttl++;
 			hpoll = sys_poll;
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 		} else {
 
 			/*
@@ -151,16 +151,17 @@ transmit(
 				peer->unreach++;
 			} else if (!(peer->flags & FLAG_CONFIG)) {
 				unpeer(peer);
-				clock_select();
 				return;
-
 			} else {
 				peer_clear(peer);
 				hpoll++;
 			}
 		}
+		if (peer->reach & 0x80)
+			peer->flags |= FLAG_CLUST;
 		oreach = peer->reach;
 		peer->reach <<= 1;
+		peer->hyst *= HYST_TC;
 		if (peer->reach == 0) {
 
 			/*
@@ -172,11 +173,9 @@ transmit(
 				peer->timereachable = current_time;
 				if (!(peer->flags & FLAG_CONFIG)) {
 					unpeer(peer);
-					clock_select();
 					return;
 				} else {
 					peer_clear(peer);
-					hpoll = peer->minpoll;
 				}
 			}
 			if (peer->flags & FLAG_IBURST)
@@ -220,9 +219,9 @@ transmit(
 			 */
 			if (peer->cast_flags & MDF_BCLNT) {
 				peer->hmode = MODE_BCLIENT;
-#ifdef AUTOKEY
+#ifdef OPENSSL
 				key_expire(peer);
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 			}
 			poll_update(peer, hpoll);
 			clock_select();
@@ -273,20 +272,23 @@ receive(
 	struct recvbuf *rbufp
 	)
 {
-	register struct peer *peer;
-	register struct pkt *pkt;
-	int hismode;
-	int oflags;
-	int restrict_mask;
-	int has_mac;			/* length of MAC field */
-	int authlen;			/* offset of MAC field */
-	int is_authentic;		/* cryptosum ok */
-	keyid_t skeyid;			/* cryptographic keys */
+	register struct peer *peer;	/* peer structure pointer */
+	register struct pkt *pkt;	/* receive packet pointer */
+	int	hismode;		/* packet mode */
+	int	oflags;			/* temp flags */
+	int	restrict_mask;		/* restrict bits */
+	int	has_mac;		/* length of MAC field */
+	int	authlen;		/* offset of MAC field */
+	int	is_authentic;		/* cryptosum ok */
+	keyid_t	skeyid;			/* cryptographic keys */
 	struct sockaddr_in *dstadr_sin;	/* active runway */
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	keyid_t pkeyid, tkeyid;		/* cryptographic keys */
-#endif /* AUTOKEY */
-	struct peer *peer2;
+	l_fp	p_org;			/* originate timestamp */
+	int	is_org;			/* loopback ok */
+	struct autokey *ap;		/* autokey structure pointer */
+	struct peer *peer2;		/* aux peer structure pointer */
+#endif /* OPENSSL */
 	int retcode = AM_NOMATCH;
 
 	/*
@@ -309,7 +311,11 @@ receive(
 #endif
 	if (restrict_mask & RES_IGNORE)
 		return;				/* no anything */
-
+	if (!(SRCPORT(&rbufp->recv_srcadr) == NTP_PORT ||
+	    SRCPORT(&rbufp->recv_srcadr) >= IPPORT_RESERVED)) {
+		sys_badlength++;
+		return;				/* invalid port */
+	}
 	pkt = &rbufp->recv_pkt;
 	if (PKT_VERSION(pkt->li_vn_mode) == NTP_VERSION) {
 		sys_newversionpkt++;		/* new version */
@@ -339,21 +345,38 @@ receive(
 	}
 
 	/*
-	 * Validate mode. Note that NTPv1 is no longer supported.
+	 * Figure out his mode and validate the packet. This has some
+	 * legacy raunch that probably should be removed. If from NTPv1
+	 * mode zero, The NTPv4 mode is determined from the source port.
+	 * If the port number is zero, it is from a symmetric active
+	 * association, which is not supported in NTPv4. Otherwise, it
+	 * is from a client association and we fake it.
 	 */
 	hismode = (int)PKT_MODE(pkt->li_vn_mode);
 	if (hismode == MODE_UNSPEC) {
-		sys_badlength++;
-		return;				/* invalid mode */
+		if (PKT_VERSION(pkt->li_vn_mode) == NTP_OLDVERSION) {
+			hismode = MODE_CLIENT;
+		} else {
+			sys_badlength++;
+			return;                 /* invalid mode */
+		}
 	}
 
 	/*
-	 * Discard broadcast packets received on the wildcard interface
-	 * or if not enabled as broadcast client.
+	 * Discard broadcast if not enabled as broadcast client. If
+	 * Autokey, the wildcard interface cannot be used, so dump
+	 * packets gettiing off the bus at that stop as well. This means
+	 * that some systems with broken interface code, specifically
+	 * Linux, will not work with Autokey.
 	 */
-	if (PKT_MODE(pkt->li_vn_mode) == MODE_BROADCAST &&
-	    (rbufp->dstadr == any_interface || !sys_bclient))
-		return;
+	if (PKT_MODE(pkt->li_vn_mode) == MODE_BROADCAST) {
+		if (!sys_bclient)
+			return;			/* no client */
+#ifdef OPENSSL
+		if (crypto_flags && rbufp->dstadr == any_interface)
+			return;			/* no client */
+#endif /* OPENSSL */
+	}
 
 	/*
 	 * Parse the extension field if present. We figure out whether
@@ -367,9 +390,9 @@ receive(
 	 * runt and goes poof! with a brilliant flash.
 	 */
 	skeyid = 0;
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	pkeyid = tkeyid = 0;
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 	authlen = LEN_PKT_NOMAC;
 	while ((has_mac = rbufp->recv_length - authlen) > 0) {
 		int temp;
@@ -386,7 +409,8 @@ receive(
 		} else if (has_mac > MAX_MAC_LEN) {
 			temp = ntohl(((u_int32 *)pkt)[authlen / 4]) &
 			    0xffff;
-			if (temp < 4 || temp % 4 != 0) {
+			if (temp < 4 || temp > NTP_MAXEXTEN || temp % 4
+			    != 0) {
 				sys_badlength++;
 				return;
 			}
@@ -402,7 +426,7 @@ receive(
 	 * the game to reduce the exposure to a clogging attack. Now we
 	 * have to burn some cycles to find the association and
 	 * authenticate the packet if required. Note that we burn only
-	 * MD5 or DES cycles, again to reduce exposure. There may be no
+	 * MD5 cycles, again to reduce exposure. There may be no
 	 * matching association and that's okay.
 	 *
 	 * More on the autokey mambo. Normally the local interface is
@@ -428,7 +452,7 @@ receive(
 			    ntoa(&rbufp->recv_srcadr), hismode, retcode);
 #endif
 	} else {
-#ifdef AUTOKEY
+#ifdef OPENSSL
 		/*
 		 * For autokey modes, generate the session key
 		 * and install in the key cache. Use the socket
@@ -480,7 +504,7 @@ receive(
 				    &rbufp->recv_srcadr, dstadr_sin, 0,
 				    sys_private, 0);
 			} else {
-				pkeyid = peer->pcookie.key;
+				pkeyid = peer->pcookie;
 			}
 
 			/*
@@ -503,7 +527,7 @@ receive(
 			}
 
 		}
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 		/*
 		 * Compute the cryptosum. Note a clogging attack may
@@ -516,10 +540,10 @@ receive(
 			is_authentic = 1;
 		else
 			sys_badauth++;
-#ifdef AUTOKEY
+#ifdef OPENSSL
 		if (skeyid > NTP_MAXKEY)
 			authtrust(skeyid, 0);
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 #ifdef DEBUG
 		if (debug)
 			printf(
@@ -555,20 +579,15 @@ receive(
 		    INT_MULTICAST)) {
 	
 			/*
-			 * We are picky about responding to a
-			 * manycaster. There is no reason to respond to
-			 * a request if our time is worse than the
-			 * manycaster. We certainly don't reply if not
-			 * synchronized to proventic time.
+			 * There is no reason to respond to a request if
+			 * our time is worse than the manycaster or it
+			 * has already synchronized to us.
 			 */
-			if (sys_peer == NULL)
-				return;
-
-			/*
-			 * We don't reply if the our stratum is greater
-			 * than the manycaster.
-			 */ 
-			if (PKT_TO_STRATUM(pkt->stratum) < sys_stratum)
+			if (sys_peer == NULL ||
+			    PKT_TO_STRATUM(pkt->stratum) <
+			    sys_stratum ||
+			    rbufp->dstadr->sin.sin_addr.s_addr ==
+			    pkt->refid)
 				return;
 		}
 
@@ -584,6 +603,7 @@ receive(
 			fast_xmit(rbufp, MODE_SERVER, 0, restrict_mask);
 		return;
 
+#ifdef OPENSSL
 	case AM_MANYCAST:
 
 		/*
@@ -604,9 +624,9 @@ receive(
 		 * since the manycast servers send the server packet
 		 * immediately.
 		 */
-		if ((restrict_mask & (RES_DONTSERVE | RES_LIMITED |
-		    RES_NOPEER)) || (sys_authenticate &&
-		    !is_authentic))
+		if (crypto_flags && ((restrict_mask & (RES_DONTSERVE |
+		    RES_LIMITED | RES_NOPEER)) || (sys_authenticate &&
+		    !is_authentic)))
 			return;
 
 		peer2 = findmanycastpeer(rbufp);
@@ -621,6 +641,7 @@ receive(
 		if (peer == NULL)
 			return;
 		break;
+#endif /* OPENSSL */
 
 	case AM_NEWPASS:
 
@@ -664,14 +685,15 @@ receive(
 		    sys_minpoll, NTP_MAXDPOLL, FLAG_MCAST |
 		    FLAG_IBURST | (sys_authenticate ?
 		    FLAG_AUTHENABLE : 0), MDF_BCLNT, 0, skeyid);
-#ifdef AUTOKEY
-#ifdef PUBKEY
 		if (peer == NULL)
 			return;
-		if (peer->flags & FLAG_SKEY)
-			crypto_recv(peer, rbufp);
-#endif /* PUBKEY */
-#endif /* AUTOKEY */
+#ifdef OPENSSL
+		if (crypto_flags && (peer->flags & FLAG_SKEY)) {
+			crypto_recv(peer, rbufp, 1);
+			if (peer->flash)
+				unpeer(peer);
+		}
+#endif /* OPENSSL */
 		return;
 
 	case AM_POSSBCL:
@@ -703,103 +725,198 @@ receive(
 	 * first packet sent cannot be authenticated and we need a way
 	 * for the dude to change his mind.
 	 */
+	peer->flash = 0;
 	oflags = peer->flags;
 	peer->timereceived = current_time;
 	peer->received++;
+	if (is_authentic)
+		peer->flags |= FLAG_AUTHENTIC;
+	else
+		peer->flags &= ~FLAG_AUTHENTIC;
 	if (!(peer->flags & FLAG_CONFIG) && has_mac) {
 		peer->flags |= FLAG_AUTHENABLE;
-#ifdef AUTOKEY
-		if (skeyid > NTP_MAXKEY)
-			peer->flags |= FLAG_SKEY;
-#endif /* AUTOKEY */
+#ifdef OPENSSL
+		if (skeyid > NTP_MAXKEY) {	/* test 5 */
+			if (crypto_flags)
+				peer->flags |= FLAG_SKEY;
+			else
+				peer->flash |= TEST5; /* no crypto */
+		}
+#endif /* OPENSSL */
+	}
+	if (peer->hmode == MODE_BROADCAST &&
+	    (restrict_mask & RES_DONTTRUST))	/* test 4 */
+		peer->flash |= TEST4;		/* access denied */
+	if (peer->flash) {
+#ifdef DEBUG
+		if (debug)
+			printf("receive: denied %03x\n",
+			    peer->flash);
+#endif
+		return;
 	}
 
 	/*
 	 * A valid packet must be from an authentic and allowed source.
-	 * All packets must pass the authentication allowed tests.
 	 * Autokey authenticated packets must pass additional tests and
 	 * public-key authenticated packets must have the credentials
 	 * verified. If all tests are passed, the packet is forwarded
 	 * for processing. If not, the packet is discarded and the
 	 * association demobilized if appropriate.
 	 */
-	peer->flash = 0;
-	if (is_authentic) {
-		peer->flags |= FLAG_AUTHENTIC;
-	} else {
-		peer->flags &= ~FLAG_AUTHENTIC;
-	}
-	if (peer->hmode == MODE_BROADCAST &&
-	    (restrict_mask & RES_DONTTRUST))	/* test 4 */
-		peer->flash |= TEST4;		/* access denied */
 	if (peer->flags & FLAG_AUTHENABLE) {
 		if (!(peer->flags & FLAG_AUTHENTIC)) /* test 5 */
 			peer->flash |= TEST5;	/* auth failed */
 		else if (!(oflags & FLAG_AUTHENABLE))
 			report_event(EVNT_PEERAUTH, peer);
 	}
+
+	/*
+	 * While the association timeout eventually recovers from
+	 * protocol restarts and signature refreshes, the dance can be
+	 * rather tedious, especially in symmetric modes. So, in case of
+	 * authentication failure, we try to jitterbug on the assumption
+	 * that, if the originate timestamp in the response packet
+	 * matches the transmit timestamp in the association, the packet
+	 * is very likely a valid response to one sent earlier. See
+	 * below for more jitterbug in other error cases.
+	 */
+#ifdef OPENSSL
+	NTOHL_FP(&pkt->org, &p_org);
+	is_org = L_ISEQU(&peer->xmt, &p_org);
+#endif /* OPENSSL */
 	if (peer->flash) {
+#ifdef OPENSSL
+		switch (hismode) {
+
+		/*
+		 * In client mode if the packet matches the last one
+		 * sent and is a crypto-NAK, the server has either
+		 * restarted or refreshed the private value, so we
+		 * start over.
+		 */
+		case MODE_SERVER:
+			if (is_org && has_mac == 4 && pkt->exten[0] ==
+			    0) {
+				if (!(peer->flags & FLAG_CONFIG)) {
+					unpeer(peer);
+					return;
+				} else {
+					peer_clear(peer);
+				}
+			}
+			break;
+
+		/*
+		 * In symmetric mode if the packet matches the last one
+		 * sent, the server has either restarted or refreshed
+		 * the private value, so we start over.
+		 */
+		case MODE_ACTIVE:
+		case MODE_PASSIVE:
+			if (is_org) {
+				if (!(peer->flags & FLAG_CONFIG)) {
+					unpeer(peer);
+					return;
+				} else {
+					peer_clear(peer);
+				}
+			}
+			break;
+		}
+#endif /* OPENSSL */
 #ifdef DEBUG
 		if (debug)
-			printf("receive: bad auth %03x\n", peer->flash);
+			printf("receive: bad auth %03x\n",
+			    peer->flash);
 #endif
 		return;
 	}
 
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	/*
 	 * More autokey dance. The rules of the cha-cha are as follows:
 	 *
 	 * 1. If there is no key or the key is not auto, do nothing.
 	 *
-	 * 2. If an extension field contains a verified signature, it is
+	 * 2. If this packet is in response to the one just previously
+	 *    sent or from a broadcast server, do the extension fields.
+	 *    Otherwise, assume bogosity and bail out.
+	 *
+	 * 3. If an extension field contains a verified signature, it is
 	 *    self-authenticated and we sit the dance.
 	 *
-	 * 3. If this is a server reply, check only to see that the
+	 * 4. If this is a server reply, check only to see that the
 	 *    transmitted key ID matches the received key ID.
 	 *
-	 * 4. Check to see that one or more hashes of the current key ID
+	 * 5. Check to see that one or more hashes of the current key ID
 	 *    matches the previous key ID or ultimate original key ID
 	 *    obtained from the broadcaster or symmetric peer. If no
 	 *    match, sit the dance and wait for timeout.
 	 */
-	if (peer->flags & FLAG_SKEY) {
+	if (crypto_flags && (peer->flags & FLAG_SKEY)) {
 		peer->flash |= TEST10;
-		crypto_recv(peer, rbufp);
-		poll_update(peer, peer->hpoll);
-		if (hismode == MODE_SERVER) {
+		crypto_recv(peer, rbufp, is_org);
+		if (peer->cmmd != 0) {
+			peer->ppoll = pkt->ppoll;
+			poll_update(peer, 0);
+		}
+		if (peer->flash & TEST12) {
+			/* fall through */
+
+		} else if (hismode == MODE_SERVER) {
 			if (skeyid == peer->keyid)
 				peer->flash &= ~TEST10;
 		} else if (!peer->flash & TEST10) {
 			peer->pkeyid = skeyid;
-		} else {
+		} else if ((ap = (struct autokey *)peer->recval.ptr) !=
+		    NULL) {
 			int i;
 
 			for (i = 0; ; i++) {
 				if (tkeyid == peer->pkeyid ||
-				    tkeyid == peer->recauto.key) {
+				    tkeyid == ap->key) {
 					peer->flash &= ~TEST10;
 					peer->pkeyid = skeyid;
 					break;
 				}
-				if (i > peer->recauto.seq)
+				if (i > ap->seq)
 					break;
 				tkeyid = session_key(
 				    &rbufp->recv_srcadr, dstadr_sin,
 				    tkeyid, pkeyid, 0);
 			}
 		}
-#ifdef PUBKEY
 
 		/*
-		 * This is delicious. Ordinarily, we kick out all errors
-		 * at this point; however, in symmetric mode and just
-		 * warming up, an unsynchronized peer must inject the
-		 * timestamps, even if it fails further up the road. So,
-		 * let the dude by here, but only if the jerk is not yet
-		 * reachable. After that, he's on his own.
+		 * If errors flash at this point and loopback, clamp the
+		 * poll to minimum. If the certificate has been stored,
+		 * scratch the association. If the server is not
+		 * reachable, assume this is the first symmetric mode
+		 * packet that warmed up the association and let it by.
+		 * It will die later after leaving the originate and
+		 * receive timestamp in the dust for the reply.
 		 */
-		if (!(peer->flags & FLAG_PROVEN))
+		if (peer->flash && is_org) {
+			poll_update(peer, peer->minpoll);
+			if (peer->crypto & CRYPTO_FLAG_PROV) {
+#ifdef DEBUG
+				if (debug)
+					printf("packet: bad crypto %03x\n",
+					    peer->flash);
+#endif
+				if (!(peer->flags & FLAG_CONFIG)) {
+					unpeer(peer);
+					return;
+				} else {
+					peer_clear(peer);
+				}
+				return;
+			} else {
+				poll_update(peer, peer->minpoll);
+			}
+		}
+		if (!(peer->crypto & CRYPTO_FLAG_PROV))
 			peer->flash |= TEST11;
 		if (peer->flash && peer->reach) {
 #ifdef DEBUG
@@ -809,9 +926,8 @@ receive(
 #endif
 			return;
 		}
-#endif /* PUBKEY */
 	}
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 	/*
 	 * We have survived the gaunt. Forward to the packet routine. If
@@ -969,7 +1085,7 @@ process_packet(
 	}
 	peer->reach |= 1;
 	peer->unreach = 0;
-	poll_update(peer, peer->hpoll);
+	poll_update(peer, 0);
 
 	/*
 	 * If running in a client/server association, calculate the
@@ -1111,9 +1227,9 @@ clock_update(void)
 	}
 	if (oleap == LEAP_NOTINSYNC) {
 		report_event(EVNT_SYNCCHG, (struct peer *)0);
-#ifdef AUTOKEY
+#ifdef OPENSSL
 		expire_all();
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 	}
 	if (ostratum != sys_stratum)
 		report_event(EVNT_PEERSTCHG, (struct peer *)0);
@@ -1129,9 +1245,9 @@ poll_update(
 	int hpoll
 	)
 {
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	int oldpoll;
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 	/*
 	 * A little foxtrot to determine what controls the poll
@@ -1141,15 +1257,17 @@ poll_update(
 	 * association to ramp up the poll interval for useless sources
 	 * and to clamp it to the minimum when first starting up.
 	 */
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	oldpoll = peer->kpoll;
-#endif /* AUTOKEY */
-	if (hpoll > peer->maxpoll)
-		peer->hpoll = peer->maxpoll;
-	else if (hpoll < peer->minpoll)
-		peer->hpoll = peer->minpoll;
-	else
-		peer->hpoll = hpoll;
+#endif /* OPENSSL */
+	if (hpoll != 0) {
+		if (hpoll > peer->maxpoll)
+			peer->hpoll = peer->maxpoll;
+		else if (hpoll < peer->minpoll)
+			peer->hpoll = peer->minpoll;
+		else
+			peer->hpoll = hpoll;
+	}
 
 	/*
 	 * Bit of adventure here. If during a burst and not timeout,
@@ -1190,7 +1308,7 @@ poll_update(
 	}
 	if (peer->nextdate < current_time)
 		peer->nextdate = current_time;
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	/*
 	 * Bit of crass arrogance at this point. If the poll interval
 	 * has changed and we have a keylist, the lifetimes in the
@@ -1199,7 +1317,7 @@ poll_update(
 	 */
 	if (peer->kpoll != oldpoll)
 		key_expire(peer);
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 #ifdef DEBUG
 	if (debug > 1)
 		printf("poll_update: at %lu %s flags %04x poll %d burst %d last %lu next %lu\n",
@@ -1219,7 +1337,6 @@ peer_clear(
 	)
 {
 	register int i;
-	u_long u_rand;
 
 	/*
 	 * If cryptographic credentials have been acquired, toss them to
@@ -1235,31 +1352,32 @@ peer_clear(
 		printf("peer_clear: at %ld assoc ID %d\n", current_time,
 		    peer->associd);
 #endif
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	key_expire(peer);
-#ifdef PUBKEY
 	if (peer->keystr != NULL)
 		free(peer->keystr);
-	if (peer->pubkey.ptr != NULL)
-		free(peer->pubkey.ptr);
-	if (peer->certif.ptr != NULL)
-		free(peer->certif.ptr);
-#endif /* PUBKEY */
-#endif /* AUTOKEY */
+	if (peer->cmmd != NULL)
+		free(peer->cmmd);
+	if (peer->cinfo.ptr != NULL) {
+		cert_free((struct cert_info *)peer->cinfo.ptr);
+		peer->cinfo.ptr = NULL;
+	}
+	value_free(&peer->cinfo);
+	value_free(&peer->cookval);
+	value_free(&peer->recval);
+	value_free(&peer->tai_leap);
+	value_free(&peer->encrypt);
+	value_free(&peer->sndval);
+#endif /* OPENSSL */
 	memset(CLEAR_TO_ZERO(peer), 0, LEN_CLEAR_TO_ZERO);
 
 	/*
 	 * If he dies as a broadcast client, he comes back to life as
 	 * a broadcast client in client mode in order to recover the
-	 * initial autokey values. Note that there is no need to call
-	 * clock_select(), since the perp has already been voted off
-	 * the island at this point.
+	 * initial autokey values.
 	 */
-	if (peer->cast_flags & MDF_BCLNT) {
-		peer->flags |= FLAG_MCAST;
-		peer->hmode = MODE_CLIENT;
-	}
-	peer->flags &= ~(FLAG_AUTOKEY | FLAG_ASSOC);
+	if (peer == sys_peer)
+		sys_peer = NULL;
 	peer->estbdelay = sys_bdelay;
 	peer->hpoll = peer->kpoll = peer->minpoll;
 	peer->ppoll = peer->maxpoll;
@@ -1279,12 +1397,16 @@ peer_clear(
 	}
 
 	/*
-	 * Randomize the first poll over 1-16s to avoid bunching.
+	 * Randomize the first poll to avoid bunching.
 	 */
-	peer->update = peer->outdate = current_time;
-	u_rand = RANDOM;
-	peer->nextdate = current_time + (u_rand & ((1 <<
-	    BURST_INTERVAL1) - 1)) + 1;
+	peer->nextdate = peer->update = peer->outdate = current_time;
+	if (peer->cast_flags & MDF_BCLNT) {
+		peer->flags |= FLAG_MCAST;
+		peer->hmode = MODE_CLIENT;
+		peer->nextdate += RANDOM & ((1 << BURST_INTERVAL1) - 1);
+	} else {
+		peer->nextdate += RANDPOLL(BURST_INTERVAL2);
+	}
 }
 
 
@@ -1336,13 +1458,13 @@ clock_filter(
 	for (i = NTP_SHIFT - 1; i >= 0; i--) {
 		if (i != 0) {
 			peer->filter_disp[j] += dtemp;
-			if (peer->filter_disp[j] > MAXDISPERSE)
+			if (peer->filter_disp[j] >= MAXDISPERSE)
 				peer->filter_disp[j] = MAXDISPERSE;
 		}
 		if (peer->filter_disp[j] >= MAXDISPERSE)
 			dst[i] = MAXDISPERSE;
 		else if (peer->update - peer->filter_epoch[j] >
-		    allan_xpt)
+		    ULOGTOD(allan_xpt))
 			dst[i] = MAXDISTANCE + peer->filter_disp[j];
 		else
  			dst[i] = peer->filter_delay[j];
@@ -1387,8 +1509,7 @@ clock_filter(
 	 * normalized close to 1.0. The jitter is the mean of the square
 	 * differences relative to the lowest delay sample. If no
 	 * acceptable samples remain in the shift register, quietly
-	 * tiptoe home leaving only the
-	 * dispersion.
+	 * tiptoe home leaving only the dispersion.
 	 */
 	jit = 0;
 	peer->disp = 0;
@@ -1420,9 +1541,10 @@ clock_filter(
 
 	/*
 	 * A new sample is useful only if it is younger than the last
-	 * one used.
+	 * one used, but only if the sucker has been synchronized.
 	 */
-	if (peer->filter_epoch[k] <= peer->epoch) {
+	if (peer->filter_epoch[k] <= peer->epoch && sys_leap !=
+	    LEAP_NOTINSYNC) {
 #ifdef DEBUG
 		if (debug)
 			printf("clock_filter: discard %lu\n",
@@ -1679,9 +1801,9 @@ clock_select(void)
 				    (struct peer *)0);
 			}
 			sys_survivors = 0;
-#ifdef AUTOKEY
+#ifdef OPENSSL
 			resetmanycast();
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 			return;
 		}
 	}
@@ -1696,20 +1818,27 @@ clock_select(void)
 	 * MAXCLOCK peers, keep only the best MAXCLOCK of them. Scan the
 	 * list to find falsetickers, who leave the island immediately.
 	 * If a falseticker is not configured, his association raft is
-	 * drowned as well. We must leave at least one peer to collect
-	 * the million bucks.
+	 * drowned as well, but only if at at least eight poll intervals
+	 * have gone by. We must leave at least one peer to collect the
+	 * million bucks.
+	 *
+	 * Note the hysteresis gimmick that decreases the effective
+	 * distance for those rascals that previously have made the
+	 * final cut. This to discourage clockhopping.
 	 */
 	j = 0;
 	for (i = 0; i < nlist; i++) {
 		peer = peer_list[i];
 		if (nlist > 1 && (low >= peer->offset || peer->offset >=
 		    high)) {
-			if (!(peer->flags & FLAG_CONFIG))
+			if (!(peer->flags & FLAG_CONFIG) &&
+			    peer->flags & FLAG_CLUST)
 				unpeer(peer);
 			continue;
 		}
 		peer->status = CTL_PST_SEL_DISTSYSPEER;
-		d = root_distance(peer) + peer->stratum * MAXDISPERSE;
+		d = (1. - peer->hyst) * (root_distance(peer) +
+		    peer->stratum * MAXDISTANCE);
 		if (j >= NTP_MAXCLOCK) {
 			if (d >= synch[j - 1])
 				continue;
@@ -1734,8 +1863,9 @@ clock_select(void)
 
 #ifdef DEBUG
 		if (debug > 2)
-			printf("select: %s distance %.6f\n",
-			    ntoa(&peer_list[i]->srcadr), synch[i]);
+			printf("select: %s stratum %d weight %.6f error %.6f\n",
+			    ntoa(&peer_list[i]->srcadr), peer->stratum,
+			    synch[i], SQRT(error[i]));
 #endif
 	}
 
@@ -1770,15 +1900,16 @@ clock_select(void)
 			}
 		}
 
-#ifdef DEBUG
-		if (debug > 2)
-			printf(
-			    "select: survivors %d select %.6f peer %.6f\n",
-			    k, SQRT(sys_selerr), SQRT(d));
-#endif
 		if (nlist <= NTP_MINCLOCK || sys_selerr <= d ||
 		    peer_list[k]->flags & FLAG_PREFER)
 			break;
+#ifdef DEBUG
+		if (debug > 2)
+			printf(
+			    "select: %s select %.6f error %.6f\n",
+			    ntoa(&peer_list[k]->srcadr),
+			    SQRT(sys_selerr), SQRT(d));
+#endif
 		if (!(peer_list[k]->flags & FLAG_CONFIG))
 			unpeer(peer_list[k]);
 		for (j = k + 1; j < nlist; j++) {
@@ -1788,7 +1919,7 @@ clock_select(void)
 		nlist--;
 	}
 
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	/*
 	 * In manycast client mode we may have spooked a sizeable number
 	 * of servers that we don't need. If there are at least
@@ -1800,14 +1931,14 @@ clock_select(void)
 	 */
 	if (sys_survivors >= NTP_MINCLOCK && nlist < NTP_MINCLOCK)
 		resetmanycast();
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 	sys_survivors = nlist;
 
 #ifdef DEBUG
-	if (debug > 2) {
+	if (debug > 1) {
 		for (i = 0; i < nlist; i++)
 			printf(
-			    "select: %s offset %.6f, distance %.6f poll %d\n",
+			    "select: %s offset %.6f, weight %.6f poll %d\n",
 			    ntoa(&peer_list[i]->srcadr),
 			    peer_list[i]->offset, synch[i],
 			    peer_list[i]->pollsw);
@@ -1843,7 +1974,7 @@ clock_select(void)
 		peer = peer_list[i];
 		peer->status = CTL_PST_SEL_SYNCCAND;
 		peer->flags |= FLAG_SYSPEER;
-		poll_update(peer, peer->hpoll);
+		peer->hyst = HYST;
 		if (peer->stratum == peer_list[0]->stratum) {
 			leap_consensus |= peer->leap;
 			if (peer->refclktype == REFCLK_ATOM_PPS &&
@@ -2015,9 +2146,9 @@ peer_xmit(
 	 * must be authenticated. If autokey is enabled, fuss with the
 	 * various modes; otherwise, private key cryptography is used.
 	 */
-#ifdef AUTOKEY
-	if ((peer->flags & FLAG_SKEY)) {
-		u_int cmmd;
+#ifdef OPENSSL
+	if (crypto_flags && (peer->flags & FLAG_SKEY)) {
+		u_int32	cmmd;
 
 		/*
 		 * The Public Key Dance (PKD): Cryptographic credentials
@@ -2077,156 +2208,111 @@ peer_xmit(
 		switch (peer->hmode) {
 
 		/*
-		 * In broadcast mode the autokey values are required.
-		 * Send them when a new keylist is generated; otherwise,
-		 * send the association ID so the client can request
-		 * them at other times.
+		 * In broadcast server mode the autokey values are
+		 * required by the broadcast clients. Push them when a
+		 * new keylist is generated; otherwise, push the
+		 * association message so the client can request them at
+		 * other times.
 		 */
 		case MODE_BROADCAST:
 			if (peer->flags & FLAG_ASSOC)
 				cmmd = CRYPTO_AUTO | CRYPTO_RESP;
 			else
 				cmmd = CRYPTO_ASSOC | CRYPTO_RESP;
-			sendlen += crypto_xmit((u_int32 *)&xpkt,
-			    sendlen, cmmd, 0, peer->associd);
+			cmmd = htonl(cmmd);
+			sendlen += crypto_xmit(&xpkt, sendlen, &cmmd, 0,
+			    peer->associd);
 			break;
 
 		/*
-		 * In symmetric modes the public key, leapsecond table,
-		 * agreement parameters and autokey values are required.
- 		 *
-		 * 1. If a response is pending, always send it first.
-		 *
-		 * 2. Don't send anything except a public-key request
-		 *    until the public key has been stored. 
-		 *
-		 * 3. Once the public key has been stored, don't send
-		 *    anything except an agreement parameter request
-		 *    until the agreement parameters have been stored.
-		 *
-		 * 4. Once the argeement parameters have been stored,
-		 *    don't send anything except a public value request
-		 *    until the agreed key has been stored.
-		 *
-		 * 5. When the agreed key has been stored and the key
-		 *    list is regenerated, send the autokey values
-		 *    gratis unless they have already been sent.
+		 * In symmetric modes the digest, certificate, agreement
+		 * parameters, cookie and autokey values are required.
+		 * The leapsecond table is optional. But, a passive peer
+		 * will not believe the active peer until the latter has
+		 * synchronized, so the agreement must be postponed
+		 * until then. In any case, if a new keylist is
+		 * generated, the autokey values are pushed.
 		 */
 		case MODE_ACTIVE:
 		case MODE_PASSIVE:
-#ifdef PUBKEY
-			if (peer->cmmd != 0)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, (peer->cmmd >> 16) |
-				    CRYPTO_RESP, peer->hcookie,
-				    peer->associd);
+			cmmd = 0;
+			if (peer->cmmd != NULL) {
+				sendlen += crypto_xmit(&xpkt, sendlen,
+				    peer->cmmd, 0, peer->associd);
+				free(peer->cmmd);
+				peer->cmmd = NULL;
+			}
 			if (!peer->crypto)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_ASSOC,
-				    peer->hcookie, peer->assoc);
-			else if (!crypto_flags &&
-			    peer->pcookie.tstamp == 0 && sys_leap !=
-			    LEAP_NOTINSYNC)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_PRIV, peer->hcookie,
-				    peer->assoc);
-			else if (crypto_flags && peer->pubkey.ptr ==
-			    NULL)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_NAME, peer->hcookie,
-				    peer->assoc);
-			else if (peer->crypto & CRYPTO_FLAG_CERT)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_CERT, peer->hcookie,
-				    peer->assoc);
-			else if (crypto_flags && peer->crypto &
-			    CRYPTO_FLAG_DH && sys_leap !=
-			    LEAP_NOTINSYNC)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_DHPAR,
-				    peer->hcookie, peer->assoc);
-			else if (crypto_flags && peer->pcookie.tstamp ==
-			    0 && sys_leap != LEAP_NOTINSYNC)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_DH, peer->hcookie,
-				    peer->assoc);
-#else
-			if (peer->cmmd != 0)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, (peer->cmmd >> 16) |
-				    CRYPTO_RESP, peer->hcookie,
-				    peer->associd);
-			if (peer->pcookie.tstamp == 0 && sys_leap !=
-			    LEAP_NOTINSYNC)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_PRIV, peer->hcookie,
-				    peer->assoc);
-#endif /* PUBKEY */
-			else if (!(peer->flags & FLAG_AUTOKEY))
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_AUTO, peer->hcookie,
-				    peer->assoc);
-			else if ((peer->flags & FLAG_ASSOC) &&
-			    (peer->cmmd >> 16) != CRYPTO_AUTO)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_AUTO | CRYPTO_RESP,
-				    peer->hcookie, peer->associd);
-#ifdef PUBKEY
-			else if (peer->crypto & CRYPTO_FLAG_TAI)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_TAI, peer->hcookie,
-				    peer->assoc);
-#endif /* PUBKEY */
-			peer->cmmd = 0;
+				cmmd = CRYPTO_ASSOC;
+			else if (!(peer->crypto & CRYPTO_FLAG_PROV))
+				cmmd = CRYPTO_CERT;
+			else if (!(peer->crypto & CRYPTO_FLAG_AGREE) &&
+			    sys_leap != LEAP_NOTINSYNC)
+				cmmd = CRYPTO_COOK;
+			else if (peer->flags & FLAG_ASSOC)
+				cmmd = CRYPTO_AUTO | CRYPTO_RESP;
+			else if (!(peer->crypto & CRYPTO_FLAG_AUTO))
+				cmmd = CRYPTO_AUTO;
+			else if (peer->crypto & CRYPTO_FLAG_TAI &&
+			    !(peer->crypto & CRYPTO_FLAG_LEAP) &&
+			    sys_leap != LEAP_NOTINSYNC)
+				cmmd = CRYPTO_TAI;
+			if (cmmd != 0) {
+				if (cmmd & CRYPTO_RESP) {
+					cmmd = htonl(cmmd);
+					sendlen += crypto_xmit(&xpkt,
+					    sendlen, &cmmd, 0,
+					    peer->associd);
+				} else {
+					cmmd = htonl(cmmd);
+					sendlen += crypto_xmit(&xpkt,
+					    sendlen, &cmmd, 0,
+					    peer->assoc);
+				}
+			}
 			break;
 
 		/*
-		 * In client mode, the public key, host cookie and
-		 * autokey values are required. In broadcast client
-		 * mode, these values must be acquired during the
+		 * In client mode the digest, certificate, agreement
+		 * parameters and cookie are required. The leapsecond
+		 * table is optional. If broadcast client mode, the
+		 * autokey values are required as well. In broadcast
+		 * client mode, these values must be acquired during the
 		 * client/server exchange to avoid having to wait until
 		 * the next key list regeneration. Otherwise, the poor
 		 * dude may die a lingering death until becoming
-		 * unreachable and attempting rebirth. Note that we ask
-		 * for the cookie at each key list regeneration anyway.
-		 */
+		 * unreachable and attempting rebirth.
+		 *
+		 * If neither the server or client have the agreement
+		 * parameters, the protocol transmits the cookie in the
+		 * clear. If the server has the parameters, the client
+		 * requests them and the protocol blinds it using the
+		 * agreed key. It is a protocol error if the client has
+		 * the parameters but the server does not.
+ 		 */
 		case MODE_CLIENT:
-			if (peer->cmmd != 0)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, (peer->cmmd >> 16) |
-				    CRYPTO_RESP, peer->hcookie,
-				    peer->associd);
+			cmmd = 0;
+			if (peer->cmmd != NULL) {
+				sendlen += crypto_xmit(&xpkt, sendlen,
+				    peer->cmmd, 0, peer->associd);
+				free(peer->cmmd);
+				peer->cmmd = NULL;
+			}
 			if (!peer->crypto)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_ASSOC,
-				    peer->hcookie, peer->assoc);
-#ifdef PUBKEY
-			else if (crypto_flags && peer->pubkey.ptr ==
-			    NULL)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_NAME, peer->hcookie,
-				    peer->assoc);
-			else if (peer->crypto & CRYPTO_FLAG_CERT)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_CERT, peer->hcookie,
-				    peer->assoc);
-#endif /* PUBKEY */
-			else if (peer->pcookie.tstamp == 0)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_PRIV, peer->hcookie,
-				    peer->assoc);
-			else if (!(peer->flags & FLAG_AUTOKEY) &&
+				cmmd = CRYPTO_ASSOC;
+			else if (!(peer->crypto & CRYPTO_FLAG_PROV))
+				cmmd = CRYPTO_CERT;
+			else if (!(peer->crypto & CRYPTO_FLAG_AGREE))
+				cmmd = CRYPTO_COOK;
+			else if (!(peer->crypto & CRYPTO_FLAG_AUTO) &&
 			    (peer->cast_flags & MDF_BCLNT))
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_AUTO, peer->hcookie,
-				    peer->assoc);
-#ifdef PUBKEY
-			else if (peer->crypto & CRYPTO_FLAG_TAI)
-				sendlen += crypto_xmit((u_int32 *)&xpkt,
-				    sendlen, CRYPTO_TAI, peer->hcookie,
-				    peer->assoc);
-#endif /* PUBKEY */
-			peer->cmmd = 0;
+				cmmd = CRYPTO_AUTO;
+			else if (peer->crypto & CRYPTO_FLAG_TAI &&
+			    !(peer->crypto & CRYPTO_FLAG_LEAP))
+				cmmd = CRYPTO_TAI;
+			if ((cmmd = htonl(cmmd)) != 0)
+				sendlen += crypto_xmit(&xpkt, sendlen,
+				    &cmmd, 0, peer->assoc);
 			break;
 		}
 
@@ -2239,7 +2325,7 @@ peer_xmit(
 			session_key(&peer->dstadr->sin, &peer->srcadr,
 			    xkeyid, 0, 2);
 	} 
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 	xkeyid = peer->keyid;
 	get_systime(&peer->xmt);
 	L_ADD(&peer->xmt, &sys_authdelay);
@@ -2247,15 +2333,16 @@ peer_xmit(
 	authlen = authencrypt(xkeyid, (u_int32 *)&xpkt, sendlen);
 	if (authlen == 0) {
 		msyslog(LOG_NOTICE,
-			"transmit: no encryption key found");
+			"transmit: encryption key %d not found",
+			    xkeyid);
 		peer->flash |= TEST4 | TEST5;
 		return;
 	}
 	sendlen += authlen;
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	if (xkeyid > NTP_MAXKEY)
 		authtrust(xkeyid, 0);
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 	get_systime(&xmt_tx);
 	if (sendlen > sizeof(xpkt)) {
 		msyslog(LOG_ERR, "buffer overflow %u", sendlen);
@@ -2276,14 +2363,14 @@ peer_xmit(
 	else
 		sys_authdelay.l_uf = sys_authdly[1];
 	peer->sent++;
-#ifdef AUTOKEY
+#ifdef OPENSSL
 #ifdef DEBUG
 	if (debug)
 		printf(
 		    "transmit: at %ld %s->%s mode %d keyid %08x len %d mac %d index %d\n",
 		    current_time, ntoa(&peer->dstadr->sin),
-		    ntoa(&peer->srcadr), peer->hmode, xkeyid, sendlen,
-		    authlen, peer->keynumber);
+		    ntoa(&peer->srcadr), peer->hmode, xkeyid, sendlen -
+		    authlen, authlen, peer->keynumber);
 #endif
 #else
 #ifdef DEBUG
@@ -2291,10 +2378,10 @@ peer_xmit(
 		printf(
 		    "transmit: at %ld %s->%s mode %d keyid %08x len %d mac %d\n",
 		    current_time, ntoa(&peer->dstadr->sin),
-		    ntoa(&peer->srcadr), peer->hmode, xkeyid, sendlen,
-		    authlen);
+		    ntoa(&peer->srcadr), peer->hmode, xkeyid, sendlen -
+		    authlen, authlen);
 #endif
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 }
 
 
@@ -2383,10 +2470,10 @@ fast_xmit(
 	 * generate the cookie, which is unique for every source-
 	 * destination-key ID combination.
 	 */
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	if (xkeyid > NTP_MAXKEY) {
 		keyid_t cookie;
-		u_int code, associd;
+		u_int associd;
 
 		/*
 		 * The only way to get here is a reply to a legitimate
@@ -2397,7 +2484,6 @@ fast_xmit(
 		 * jerk can decode it. If no extension field is present,
 		 * use the cookie to generate the session key.
 		 */
-		code = (htonl(rpkt->exten[0]) >> 16) | CRYPTO_RESP;
 		cookie = session_key(&rbufp->recv_srcadr,
 		    &rbufp->dstadr->sin, 0, sys_private, 0);
 		associd = htonl(rpkt->exten[1]);
@@ -2405,23 +2491,24 @@ fast_xmit(
 		    sizeof(u_int32)) {
 			session_key(&rbufp->dstadr->sin,
 			    &rbufp->recv_srcadr, xkeyid, 0, 2);
-			sendlen += crypto_xmit((u_int32 *)&xpkt,
-			    sendlen, code, cookie, associd);
+			*rpkt->exten |= htonl(CRYPTO_RESP);
+			sendlen += crypto_xmit(&xpkt, sendlen,
+			    rpkt->exten, cookie, associd);
 		} else {
 			session_key(&rbufp->dstadr->sin,
 			    &rbufp->recv_srcadr, xkeyid, cookie, 2);
 		}
 	}
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 	get_systime(&xmt_ts);
 	L_ADD(&xmt_ts, &sys_authdelay);
 	HTONL_FP(&xmt_ts, &xpkt.xmt);
 	authlen = authencrypt(xkeyid, (u_int32 *)&xpkt, sendlen);
 	sendlen += authlen;
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	if (xkeyid > NTP_MAXKEY)
 		authtrust(xkeyid, 0);
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 	get_systime(&xmt_tx);
 	if (sendlen > sizeof(xpkt)) {
 		msyslog(LOG_ERR, "buffer overflow %u", sendlen);
@@ -2446,13 +2533,13 @@ fast_xmit(
 		printf(
 		    "transmit: at %ld %s->%s mode %d keyid %08x len %d mac %d\n",
 		    current_time, ntoa(&rbufp->dstadr->sin),
-		    ntoa(&rbufp->recv_srcadr), xmode, xkeyid, sendlen,
-		    authlen);
+		    ntoa(&rbufp->recv_srcadr), xmode, xkeyid, sendlen -
+		    authlen, authlen);
 #endif
 }
 
 
-#ifdef AUTOKEY
+#ifdef OPENSSL
 /*
  * key_expire - purge the key list
  */
@@ -2469,13 +2556,14 @@ key_expire(
 		free(peer->keylist);
 		peer->keylist = NULL;
 	}
-	peer->keynumber = peer->sndauto.seq = 0;
+	value_free(&peer->sndval);
+	peer->keynumber = 0;
 #ifdef DEBUG
 	if (debug)
 		printf("key_expire: at %lu\n", current_time);
 #endif
 }
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 /*
  * Find the precision of this particular machine
@@ -2502,8 +2590,7 @@ int
 default_get_precision(void)
 {
 	struct timeval tp;
-#if !defined(SYS_WINNT) && !defined(VMS) && !defined(_SEQUENT_) && \
-    !defined(MPE)
+#if !defined(SYS_WINNT) && !defined(VMS) && !defined(_SEQUENT_)
 	struct timezone tzp;
 #elif defined(VMS) || defined(_SEQUENT_)
 	struct timezone {
@@ -2618,9 +2705,9 @@ init_proto(void)
 	sys_processed = 0;
 	sys_badauth = 0;
 	sys_manycastserver = 0;
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	sys_automax = 1 << NTP_AUTOMAX;
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 	/*
 	 * Default these to enable
