@@ -11,9 +11,7 @@
 
 #include <stdio.h>
 #include <signal.h>
-#ifdef HAVE_SYS_SIGNAL_H
-# include <sys/signal.h>
-#endif
+#include <sys/signal.h>
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -23,9 +21,9 @@
 # include "ntp_timer.h"
 #endif
 
-#ifdef PUBKEY
+#ifdef OPENSSL
 #include "ntp_crypto.h"
-#endif /* PUBKEY */
+#endif /* OPENSSL */
 
 /*
  * These routines provide support for the event timer.	The timer is
@@ -50,10 +48,10 @@ static	u_long adjust_timer;		/* second timer */
 static	u_long keys_timer;		/* minute timer */
 static	u_long hourly_timer;		/* hour timer */
 static	u_long huffpuff_timer;		/* huff-n'-puff timer */
-#ifdef AUTOKEY
+#ifdef OPENSSL
 static	u_long revoke_timer;		/* keys revoke timer */
-u_long	sys_revoke = 1 << KEY_REVOKE;	/* keys revoke timeout */
-#endif /* AUTOKEY */
+u_char	sys_revoke = KEY_REVOKE;	/* keys revoke timeout (log2 s) */
+#endif /* OPENSSL */
 
 /*
  * Statistics counter for the interested.
@@ -218,6 +216,9 @@ void
 timer(void)
 {
 	register struct peer *peer, *next_peer;
+#ifdef OPENSSL
+	u_char	statstr[NTP_MAXSTRLEN]; /* statistics for filegen */
+#endif /* OPENSSL */
 	u_int n;
 
 	current_time += (1<<EVENT_TIMEOUT);
@@ -269,20 +270,21 @@ timer(void)
 		huffpuff();
 	}
 
-#ifdef AUTOKEY
+#ifdef OPENSSL
 	/*
 	 * Garbage collect old keys and generate new private value
 	 */
 	if (revoke_timer <= current_time) {
-		revoke_timer += sys_revoke;
+		revoke_timer += RANDPOLL(sys_revoke);
 		expire_all();
+		sprintf(statstr, "refresh ts %u", ntohl(host.tstamp));
+		record_crypto_stats(NULL, statstr);
 #ifdef DEBUG
 		if (debug)
-			printf("key expire: at %lu next %lu\n",
-			    current_time, revoke_timer);
+			printf("timer: %s\n", statstr);
 #endif
 	}
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 
 	/*
 	 * Finally, call the hourly routine.
