@@ -6,7 +6,6 @@
 # include <config.h>
 #endif
 
-#include "ntp_machine.h"
 #include "ntpd.h"
 #include "ntp_io.h"
 #include "ntp_stdlib.h"
@@ -20,15 +19,9 @@
 #include <stdio.h>
 #ifndef SYS_WINNT
 # if !defined(VMS)	/*wjm*/
-#  ifdef HAVE_SYS_PARAM_H
-#   include <sys/param.h>
-#  endif
+#  include <sys/param.h>
 # endif /* VMS */
-# ifdef HAVE_SYS_SIGNAL_H
-#  include <sys/signal.h>
-# else
-#  include <signal.h>
-# endif
+# include <sys/signal.h>
 # ifdef HAVE_SYS_IOCTL_H
 #  include <sys/ioctl.h>
 # endif /* HAVE_SYS_IOCTL_H */
@@ -99,9 +92,9 @@
 # include <sys/ci/ciioctl.h>
 #endif
 
-#ifdef PUBKEY
+#ifdef OPENSSL
 #include "ntp_crypto.h"
-#endif /* PUBKEY */
+#endif /* OPENSSL */
 
 /*
  * Signals we catch for debugging.	If not debugging we ignore them.
@@ -125,7 +118,7 @@ HANDLE ResolverThreadHandle = NULL;
 /* variables used to inform the Service Control Manager of our current state */
 SERVICE_STATUS ssStatus;
 SERVICE_STATUS_HANDLE	sshStatusHandle;
-HANDLE WaitHandles[3] = { NULL, NULL, NULL };
+HANDLE WaitHandles[2] = { NULL, NULL };
 char szMsgPath[255];
 static BOOL WINAPI OnConsoleEvent(DWORD dwCtrlType);
 #endif /* SYS_WINNT */
@@ -335,10 +328,6 @@ ntpdmain(
 {
 	l_fp now;
 	char *cp;
-#ifdef AUTOKEY
-	u_int n;
-	char hostname[MAXFILENAME];
-#endif /* AUTOKEY */
 	struct recvbuf *rbuflist;
 	struct recvbuf *rbuf;
 #ifdef _AIX			/* HMS: ifdef SIGDANGER? */
@@ -361,7 +350,7 @@ ntpdmain(
 	}
 #endif
 
-#if defined(HAVE_GETUID) && !defined(MPE) /* MPE lacks the concept of root */
+#ifdef HAVE_GETUID
 	{
 		uid_t uid;
 
@@ -384,9 +373,6 @@ ntpdmain(
 #endif
 	getstartup(argc, argv); /* startup configuration, may set debug */
 
-	if (debug)
-	    printf("%s\n", Version);
-
 	/*
 	 * Initialize random generator and public key pair
 	 */
@@ -417,12 +403,12 @@ ntpdmain(
 			int max_fd;
 #endif /* not F_CLOSEM */
 
-#if defined(F_CLOSEM)
 			/*
 			 * From 'Writing Reliable AIX Daemons,' SG24-4946-00,
 			 * by Eric Agar (saves us from doing 32767 system
 			 * calls)
 			 */
+#if defined(F_CLOSEM)
 			if (fcntl(0, F_CLOSEM, 0) == -1)
 			    msyslog(LOG_ERR, "ntpd: failed to close open files(): %m");
 #else  /* not F_CLOSEM */
@@ -523,10 +509,6 @@ service_main(
 	char *cp;
 	struct recvbuf *rbuflist;
 	struct recvbuf *rbuf;
-#ifdef AUTOKEY
-	u_int n;
-	char hostname[MAXFILENAME];
-#endif /* AUTOKEY */
 	if(!debug)
 	{
 		/* register our service control handler */
@@ -744,15 +726,9 @@ service_main(
 	debug = 0;
 #endif
 	getconfig(argc, argv);
-#ifdef AUTOKEY
-	gethostname(hostname, MAXFILENAME);
-	n = strlen(hostname) + 1;
-	sys_hostname = emalloc(n);
-	memcpy(sys_hostname, hostname, n);
-#ifdef PUBKEY
+#ifdef OPENSSL
 	crypto_setup();
-#endif /* PUBKEY */
-#endif /* AUTOKEY */
+#endif /* OPENSSL */
 	initializing = 0;
 
 #if defined(SYS_WINNT) && !defined(NODETACH)
@@ -800,7 +776,6 @@ service_main(
 #if defined(HAVE_IO_COMPLETION_PORT)
 		WaitHandles[0] = CreateEvent(NULL, FALSE, FALSE, NULL); /* exit reques */
 		WaitHandles[1] = get_timer_handle();
-	    WaitHandles[2] = get_io_event();
 
 		for (;;) {
 			DWORD Index = WaitForMultipleObjectsEx(sizeof(WaitHandles)/sizeof(WaitHandles[0]), WaitHandles, FALSE, 1000, MWMO_ALERTABLE);
@@ -812,36 +787,16 @@ service_main(
 				case WAIT_OBJECT_0 + 1 : /* timer */
 					timer();
 				break;
-
-				case WAIT_OBJECT_0 + 2 : /* Io event */
-# ifdef DEBUG
-					if ( debug > 3 )
-					{
-						printf( "IoEvent occurred\n" );
-					}
-# endif
-				break;
-
-# if 1
-				/*
-				 * FIXME: According to the documentation for WaitForMultipleObjectsEx
-				 *        this is not possible. This may be a vestigial from when this was
-				 *        MsgWaitForMultipleObjects, maybe it should be removed?
-				 */
-				case WAIT_OBJECT_0 + 3 : /* windows message */
-				{
+				case WAIT_OBJECT_0 + 2 : { /* Windows message */
 					MSG msg;
-					while ( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
-					{
-						if ( msg.message == WM_QUIT )
-						{
-							exit( 0 );
+					while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
+						if (msg.message == WM_QUIT) {
+							exit(0);
 						}
-						DispatchMessage( &msg );
+						DispatchMessage(&msg);
 					}
 				}
 				break;
-# endif
 
 				case WAIT_IO_COMPLETION : /* loop */
 				case WAIT_TIMEOUT :
@@ -903,10 +858,9 @@ service_main(
 			}
 			else if (nfound == -1 && errno != EINTR)
 				msyslog(LOG_ERR, "select() error: %m");
-#  ifdef DEBUG
-			else if (debug > 2)
+			else if (debug > 2) {
 				msyslog(LOG_DEBUG, "select(): nfound=%d, error: %m", nfound);
-#  endif /* DEBUG */
+			}
 # else /* HAVE_SIGNALED_IO */
                         
 			wait_for_signal();
