@@ -45,12 +45,11 @@ static	const char *	prompt = "ntpdc> ";	/* prompt to ask him about */
 static	u_long	info_auth_keyid;
 
 /*
- * Type of key md5 or des
+ * Type of key md5
  */
-#define	KEY_TYPE_DES	3
 #define	KEY_TYPE_MD5	4
 
-static	int info_auth_keytype = KEY_TYPE_MD5;	/* MD5*/
+static	int info_auth_keytype = KEY_TYPE_MD5;	/* MD5 */
 u_long	current_time;		/* needed by authkeys; not used */
 
 int		ntpdcmain	P((int,	char **));
@@ -191,11 +190,6 @@ void timer(void)	{  ; };	/* 1998/06/03 - Used in ntplib/machines.c */
 static	char *pktdata;
 static	int pktdatasize;
 
-/*
- * These are used to help the magic with old and new versions of ntpd.
- */
-static int req_pkt_size = REQ_LEN_NOMAC;
-
 /*
  * For commands typed on the command line (with the -c option)
  */
@@ -479,7 +473,6 @@ openhost(
 	    error("connect", "", "");
 	
 	havehost = 1;
-	req_pkt_size = REQ_LEN_NOMAC;
 	return 1;
 }
 
@@ -776,15 +769,11 @@ sendrequest(
 
 	if (!auth) {
 		qpkt.auth_seq = AUTH_SEQ(0, 0);
-		return sendpkt((char *)&qpkt, req_pkt_size);
+		return sendpkt((char *)&qpkt, REQ_LEN_NOMAC);
 	} else {
 		l_fp ts;
 		int maclen = 0;
 		const char *pass = "\0";
-		struct req_pkt_tail *qpktail;
-
-		qpktail = (struct req_pkt_tail *)((char *)&qpkt + req_pkt_size
-		    + MAX_MAC_LEN - sizeof(struct req_pkt_tail));
 
 		if (info_auth_keyid == 0) {
 			maclen = getkeyid("Keyid: ");
@@ -796,8 +785,7 @@ sendrequest(
 			info_auth_keyid = maclen;
 		}
 		if (!authistrusted(info_auth_keyid)) {
-			pass = getpass((info_auth_keytype == KEY_TYPE_DES)
-			    ? "DES Password: " : "MD5 Password: ");
+			pass = getpass("MD5 Password: ");
 			if (*pass == '\0') {
 				(void) fprintf(stderr,
 				    "Invalid password\n");
@@ -807,17 +795,17 @@ sendrequest(
 		authusekey(info_auth_keyid, info_auth_keytype, (const u_char *)pass);
 		authtrust(info_auth_keyid, 1);
 		qpkt.auth_seq = AUTH_SEQ(1, 0);
-		qpktail->keyid = htonl(info_auth_keyid);
+		qpkt.keyid = htonl(info_auth_keyid);
 		get_systime(&ts);
 		L_ADD(&ts, &delay_time);
-		HTONL_FP(&ts, &qpktail->tstamp);
+		HTONL_FP(&ts, &qpkt.tstamp);
 		maclen = authencrypt(info_auth_keyid, (u_int32 *)&qpkt,
-		    req_pkt_size);
+		    REQ_LEN_NOMAC);
 		if (maclen == 0) {  
 			(void) fprintf(stderr, "Key not found\n");
 			return (1);
 		}
-		return sendpkt((char *)&qpkt, (int)(req_pkt_size + maclen));
+		return sendpkt((char *)&qpkt, (int)(REQ_LEN_NOMAC + maclen));
 	}
 	/*NOTREACHED*/
 }
@@ -856,7 +844,6 @@ doquery(
 	/*
 	 * Poll the socket and clear out any pending data
 	 */
-again:
 	do {
 		tvzero.tv_sec = tvzero.tv_usec = 0;
 		FD_ZERO(&fds);
@@ -884,26 +871,6 @@ again:
 	 */
 	res = getresponse(implcode, reqcode, ritems, rsize, rdata);
 
-	/*
-	 * Try to be compatible with older implementations of ntpd.
-	 */
-	if (res == INFO_ERR_FMT && req_pkt_size != 48) {
-		int oldsize;
-
-		oldsize = req_pkt_size;
-
-		switch(req_pkt_size) {
-		case REQ_LEN_NOMAC:
-			req_pkt_size = 48;
-			break;
-		}
-
-		fprintf(stderr,
-		    "***Warning changing the request packet size from %d to %d\n",
-		    oldsize, req_pkt_size);
-		goto again;
-	}
-
  	/* log error message if not told to be quiet */
  	if ((res > 0) && (((1 << res) & quiet_mask) == 0)) {
 		switch(res) {
@@ -1532,7 +1499,7 @@ keytype(
 {
 	if (pcmd->nargs == 0)
 	    fprintf(fp, "keytype is %s\n",
-		    (info_auth_keytype == KEY_TYPE_MD5) ? "MD5" : "DES");
+		    (info_auth_keytype == KEY_TYPE_MD5) ? "MD5" : "???");
 	else
 	    switch (*(pcmd->argval[0].string)) {
 		case 'm':
@@ -1540,13 +1507,8 @@ keytype(
 		    info_auth_keytype = KEY_TYPE_MD5;
 		    break;
 
-		case 'd':
-		case 'D':
-		    info_auth_keytype = KEY_TYPE_DES;
-		    break;
-
 		default:
-		    fprintf(fp, "keytype must be 'md5' or 'des'\n");
+		    fprintf(fp, "keytype must be 'md5'\n");
 	    }
 }
 
@@ -1576,10 +1538,7 @@ passwd(
 			   (u_char *)pcmd->argval[0].string);
 		authtrust(info_auth_keyid, 1);
 	} else {
-		pass = getpass((info_auth_keytype == KEY_TYPE_DES)
-			       ? "DES Password: "
-			       : "MD5 Password: "
-			       );
+		pass = getpass("MD5 Password: ");
 		if (*pass == '\0')
 		    (void) fprintf(fp, "Password unchanged\n");
 		else {
